@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ char *command_name = NULL; /* for usage() */
 
 char* globolize_path(char *path);
 
-static char usg[] = 
+static char usg[] =
 "Usage: %s [options] path\n"
 "\t-h\thelp\n"
 "\t-V\tversion info\n"
@@ -42,14 +42,14 @@ int main(int argc, char **argv)
 	struct scan_info info;
 
 	parse_global_options(&argc, &argv, usg);
-	
+
 	mount_point = globolize_path(argv[0]);
 	info.ugid_stat = NULL;
 	scan(&info, mount_point);
 
 	printf("quota usage for %s\n", mount_point);
 	printf("%11s%11s\n","blocks", "inodes");
-	printf("%11u%11u\n", ker2block(info.size), info.inodes);
+	printf("%11llu%11u\n", ker2block(info.size), info.inodes);
 	exit(0);
 }
 
